"""Unit tests for the Ubuntu One server adapter for Cloudspaces tests."""

import contextlib
import unittest

import mock
import requests_oauthlib
import ssoclient.v2

from cloudspacesclient import ubuntuone


class UbuntuOneServerMock(ubuntuone.UbuntuOneServer):

    def __init__(self):
        super(UbuntuOneServerMock, self).__init__()
        self.patches = []

    def start(self):
        sso_register_patch = mock.patch.object(
            ssoclient.v2.V2ApiClient, 'register')
        self.patches.append(sso_register_patch)
        self.sso_register_mock = sso_register_patch.start()
        
        sso_login_patch = mock.patch.object(ssoclient.v2.V2ApiClient, 'login')
        self.patches.append(sso_login_patch)
        self.sso_login_mock = sso_login_patch.start()
        
        create_u1_user_patch = mock.patch.object(
            ubuntuone.UbuntuOneAPIClient, 'create_user')
        self.patches.append(create_u1_user_patch)
        self.create_u1_user_mock = create_u1_user_patch.start()
        self.create_u1_user_mock.return_value = type(
            'object', (), {'ok': True})

    def stop(self):
        for patch in self.patches:
            patch.stop()

    def __enter__(self):
        self.start()        
        return self

    def __exit__(self, *args):
        self.stop()


class UbuntuOneServerTestCase(unittest.TestCase):

    def setUp(self):
        super(UbuntuOneServerTestCase, self).setUp()
        self.server = UbuntuOneServerMock()
        self.server.start()
        self.addCleanup(self.server.stop)

    def test_create_test_user_with_unique_id(self):
        user = self.server.create_test_user(unique_id='test_unique_id')
        self.assertEqual(user.full_name, 'Test user test_unique_id')
        self.assertEqual(
            user.email, 'cloudspacestest+test_unique_id@example.com')
        self.assertEqual(user.password, 'Hola123*')

    def test_create_test_user_without_unique_id(self):
        with mock.patch('uuid.uuid1') as uuid_mock:
            uuid_mock.return_value = 'test_uuid'
            user = self.server.create_test_user()

        self.assertEqual(user.full_name, 'Test user test_uuid')
        self.assertEqual(user.email, 'cloudspacestest+test_uuid@example.com')
        self.assertEqual(user.password, 'Hola123*')

    def test_get_sso_api_client(self):
        with mock.patch('cloudspacesclient.config.get') as config_mock:
            config_mock.return_value = 'http://example.com'
            client = self.server._get_sso_api_client()

        self.assertIsInstance(client, ssoclient.v2.V2ApiClient)
        self.assertEqual(client.session.endpoint, 'http://example.com/api/v2/')

    def test_create_test_user_should_register_user_in_sso(self):
        self.server.create_test_user(unique_id='id')

        self.server.sso_register_mock.assert_called_once_with(
            {
                'password': 'Hola123*',
                'displayname': 'Test user id',
                'email': 'cloudspacestest+id@example.com'
            }
        )

    def test_create_test_user_fails_to_register_user_in_sso(self):
        sso_exception = ssoclient.v2.ApiException(
            'object', (), {'status_code': 'test error code'})
        self.server.sso_register_mock.side_effect = sso_exception

        with self.assertRaises(ubuntuone.UbuntuOneServerException) as e:
            self.server.create_test_user()

        self.assertEqual(
            str(e.exception),
            'Failed to register SSO user. ' + str(sso_exception))
        self.assertFalse(self.server.create_u1_user_mock.called)

    def test_get_auth_credentials_should_return_oauth1(self):
        self.server.sso_login_mock.return_value = {
            'consumer_key': 'test consumer key',
            'consumer_secret': 'test consumer secret',
            'token_key': 'test token key',
            'token_secret': 'test token secret'
        }
        user = ubuntuone.User.make_unique(unique_id='id')

        auth = self.server.get_auth_credentials(user)

        self.server.sso_login_mock.assert_called_once_with(
            {
                'password': 'Hola123*',
                'email': 'cloudspacestest+id@example.com',
                'token_name': 'Cloudspaces API test',
            }
        )
        self.assertIsInstance(auth, requests_oauthlib.OAuth1)
        self.assertEqual(auth.client.client_key, 'test consumer key')
        self.assertEqual(auth.client.client_secret, 'test consumer secret')
        self.assertEqual(auth.client.resource_owner_key, 'test token key')
        self.assertEqual(
            auth.client.resource_owner_secret, 'test token secret')

    def test_get_u1_api_client(self):
        with contextlib.nested(
            mock.patch('cloudspacesclient.config.get'),
            mock.patch.object(self.server, 'get_auth_credentials')
        ) as (config_mock, credentials_mock):
            config_mock.return_value = 'http://example.com'
            credentials_mock.return_value = 'test credentials'                
            client = self.server._get_u1_api_client('dummy user')

        self.assertIsInstance(client, ubuntuone.UbuntuOneAPIClient)
        self.assertEqual(
            client.service_root_url, 'http://example.com/api/')
        self.assertEqual(client.auth, 'test credentials')

    def test_create_test_user_should_create_user_in_u1(self):
        self.server.create_test_user()

        self.server.create_u1_user_mock.assert_called_once_with()

    def test_create_test_user_fails_to_create_user_in_u1(self):
        self.server.create_u1_user_mock.return_value = type(
            'object', (),
            {
                'ok': False,
                'status_code': 'test error code',
                'content': 'test content'
            }
        )

        with self.assertRaises(ubuntuone.UbuntuOneServerException) as e:
            self.server.create_test_user()

        self.assertEqual(
            str(e.exception),
            'Failed to create U1 user. '
            'status_code: test error code, content: test content')
