"""Ubuntu One server adapter for Cloudspaces tests."""

import urlparse
import uuid

import requests
import requests_oauthlib
import ssoclient.v2

from cloudspacesclient import config

JSON_MIME_TYPE = 'application/json'


class UbuntuOneServerException(Exception):
    """Exception raised when there is an error with the server."""


class User(object):

    def __init__(self, full_name, email, password):
        self.full_name = full_name
        self.email = email
        self.password = password

    @classmethod
    def make_unique(cls, unique_id=None):
        """Return a unique user.

        :parameter unique_id: If a unique_id is passed, it will be used to
            uniquely identify the data of the new user. If None is passed as
            unique_id, then a uuid will be used. Default is None.

        """
        if unique_id is None:
            unique_id = str(uuid.uuid1())
        # The full name in Ubuntu One can't be more than 30 characters.
        full_name = 'Test user {}'.format(unique_id)[:30]
        email = 'cloudspacestest+{}@example.com'.format(unique_id)
        password = 'Hola123*'
        return cls(full_name, email, password)


class UbuntuOneServer(object):

    def create_test_user(self, unique_id=None):
        """Create an Ubuntu One user.

        :return: An object with the user information.

        """
        user = User.make_unique(unique_id)
        self._register_test_user(user)
        self._create_user_in_u1(user)
        return user

    def _register_test_user(self, user):
        data = dict(
            email=user.email,
            password=user.password,
            displayname=user.full_name
        )
        try:
            self._get_sso_api_client().register(data)
        except ssoclient.v2.ApiException as e:
            raise UbuntuOneServerException(
                'Failed to register SSO user. ' + str(e))

    def _get_sso_api_client(self):
        return ssoclient.v2.V2ApiClient(
            urlparse.urljoin(self._get_sso_server_url(), 'api/v2'))

    def _get_sso_server_url(self):
        return config.get('ubuntuone', 'sso_server_url')

    def _create_user_in_u1(self, user):
        response = self._get_u1_api_client(user).create_user()
        if not response.ok:
            raise UbuntuOneServerException(
                'Failed to create U1 user. '
                'status_code: {0}, content: {1}'.format(
                    response.status_code, response.content))

    def _get_u1_api_client(self, user):
        url = urlparse.urljoin(self._get_ubuntuone_server_url(), 'api/')
        auth = self.get_auth_credentials(user)
        return UbuntuOneAPIClient(url, auth)

    def _get_ubuntuone_server_url(self):
        return config.get('ubuntuone', 'ubuntuone_server_url')

    def delete_test_user(self, user):
        # Currently we can't delete users from U1 and SSO.
        pass

    def get_auth_credentials(self, user):
        """Return the auth credentials to sign the Cloudspaces API requests.

        :parameter user: An object with the user information.

        """
        data = dict(
            email=user.email,
            password=user.password,
            token_name='Cloudspaces API test')
        response = self._get_sso_api_client().login(data)
        return requests_oauthlib.OAuth1(
            response.get('consumer_key'), response.get('consumer_secret'),
            response.get('token_key'), response.get('token_secret'))


class UbuntuOneAPIClient(object):

    content_type = JSON_MIME_TYPE

    def __init__(self, service_root_url, auth):
        super(UbuntuOneAPIClient, self).__init__()
        self.service_root_url = service_root_url
        self.auth = auth

    def create_user(self):
        """Create the authenticated user in Ubuntu One, if it doesn't exist."""
        # Any call to the API will create the user if it doesn't exist.
        return self.get_account_info()

    def get_account_info(self):
        """Get the account information of the authenticated user."""
        url = urlparse.urljoin(self.service_root_url, 'account/')
        return requests.get(
            url, auth=self.auth, headers=self._get_headers())

    def _get_headers(self):
        return {
            'Content-Type': self.content_type,
            'Accept': self.content_type
        }
